#define _GNU_SOURCE
#include <alsa/asoundlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <vorbis/vorbisenc.h>
#include "ogg.h"

ogg_stream_state os; 
ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
ogg_packet       op; /* one raw packet of data for decode */

vorbis_info      vi; /* struct that stores all the static vorbis bitstream
			settings */
vorbis_comment   vc; /* struct that stores all the user comments */

vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
vorbis_block     vb; /* local working space for packet->PCM decode */

int channels, resolution, frame_size;
long sampling_rate;

int i, founddata;

FILE* fp;

int ogg_open_stream(const char* filename){
  	int ret;
	vorbis_info_init(&vi);
	ret = vorbis_encode_init_vbr(&vi,channels,sampling_rate,0.1);
	if(ret)exit(1);
	vorbis_comment_init(&vc);
	vorbis_comment_add_tag(&vc,"ENCODER","rec.c");

	vorbis_analysis_init(&vd,&vi);
	vorbis_block_init(&vd,&vb);

	srand(time(NULL));
	ogg_stream_init(&os,rand());

	{
	ogg_packet header;
	ogg_packet header_comm;
	ogg_packet header_code;


	vorbis_analysis_headerout(&vd,&vc,&header,&header_comm,&header_code);
	ogg_stream_packetin(&os,&header); /* automatically placed in its own
					     page */
	ogg_stream_packetin(&os,&header_comm);
	ogg_stream_packetin(&os,&header_code);

	}

	// open file 
	if ( (fp = fopen(filename, "w+")) == NULL)
	{
		fprintf(stderr, "Cannot open file %s!\n", filename); 
		return -3; // failed to open file
	}
	/* This ensures the actual
	 * audio data will start on a new page, as per spec
	 */
	while(ogg_stream_flush(&os,&og) != 0){
		fwrite(og.header,1,og.header_len,fp);
		fwrite(og.body,1,og.body_len,fp);
	}

	// success!
	return 0;				
}

int ogg_close_stream(){
  ogg_stream_clear(&os);
  vorbis_block_clear(&vb);
  vorbis_dsp_clear(&vd);
  vorbis_comment_clear(&vc);
  vorbis_info_clear(&vi);
  
  /* ogg_page and ogg_packet structs always point to storage in
     libvorbis.  They're never freed or manipulated directly */
  fclose(fp);  
  fprintf(stderr,"Done.\n");
  return(0);
}

void ogg_set_resolution(int res){
	resolution = res;
	frame_size = channels * resolution / 8;
}

void ogg_set_sampling_rate(long rate){
	sampling_rate = rate;
}

void ogg_set_channels(int chans){
	channels = chans;
	frame_size = channels * resolution / 8;
}

int ogg_write_buffer(char* buf, unsigned int bytes){
	int eos = 0;
	long i, j;
	if(bytes==0){
		vorbis_analysis_wrote(&vd,0);
	}else{
		/* data to encode */
		/* expose the buffer to submit data */
		float **buffer=vorbis_analysis_buffer(&vd, bytes/frame_size);

		// uninterleave samples - FIXME: assumes 16 bit resolution !!!
		     for(i = 0; i < bytes/frame_size; i++){
			     for (j = 0; j < channels; j++){
				     buffer[j][i]=((buf[i*frame_size+j*2+1]<<8)|
						     (0x00ff&(int)buf[i*frame_size+j*2]))/32768.f;
			     }
		     }


		/* tell the library how much we actually submitted */
		vorbis_analysis_wrote(&vd,i);
	}

	while(vorbis_analysis_blockout(&vd,&vb)==1){

		/* analysis, assume we want to use bitrate management */
		vorbis_analysis(&vb,NULL);
		vorbis_bitrate_addblock(&vb);

		while(vorbis_bitrate_flushpacket(&vd,&op)){

			/* weld the packet into the bitstream */
			ogg_stream_packetin(&os,&op);

			/* write out pages (if any) */
			while(!eos){
				int result=ogg_stream_pageout(&os,&og);
				if(result==0)break;
				fwrite(og.header,1,og.header_len,fp);
				fwrite(og.body,1,og.body_len,fp);

				/* this could be set above, but for illustrative purposes, I do
				   it here (to show that vorbis does know where the stream ends) */

				if(ogg_page_eos(&og))eos=1;
			}
		}
	}
	return bytes;
}
