/* rec.c
 *
 * record audio from ALSA into a Ogg Vorbis fileplay
 *
 * (C) 2006 Raphael Wimmer <raphael@hcilab.org>
 *
 * using code from http://equalarea.com/paul/alsa-audio.html
 *
 * licensed under the terms of the GNU GPL v2
 * (http://www.fsf.org/licensing/licenses/gpl.html) 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <alsa/asoundlib.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisenc.h>
#include "ogg.h"

#define BUFSIZE 1024 // Frames
#define SAMPLING_RATE 44100 // Hz
#define CHANNELS 2 // umm, Channels?
#define RESOLUTION 16 //bit

static char *device = "plughw:0,0";  
snd_pcm_t *capture_handle;
char *buf;

// SET DEFAULTS //

int frame_size = CHANNELS * RESOLUTION / 8; // bytes

unsigned int sampling_rate = SAMPLING_RATE;
unsigned int channels = CHANNELS;
unsigned int resolution = RESOLUTION;

snd_pcm_format_t snd_fmt = SND_PCM_FORMAT_S16_LE;


int init_alsa(unsigned int sampling_rate, snd_pcm_format_t snd_fmt, int channels);

int (*open_stream) (const char* file_name) = NULL;
int (*close_stream) () = NULL;
void (*set_resolution) (int res) = NULL;
void (*set_sampling_rate) (long rate) = NULL;
void (*set_channels) (int channels) = NULL;
int (*write_buffer) (char* buf, unsigned int bytes) = NULL;

void usage(char* arg0){
	printf("Usage: %s <output.ogg>\n", arg0);
}

void quit(int sig){
    if(sig == SIGINT) {printf("Terminated by user (SIGINT).\n");}
    printf("Cleaning up...\n");
    close_stream();
    snd_pcm_close (capture_handle);
    free(buf); buf = NULL;
    exit(0);
}


int main (int argc, char *argv[]) {
	int err;

	if (argc != 2){
		usage(argv[0]);
		return(0);
	}

	signal(SIGINT, quit);

	open_stream = &ogg_open_stream;
	close_stream = &ogg_close_stream;
	set_resolution = &ogg_set_resolution;
	set_sampling_rate = &ogg_set_sampling_rate;
	set_channels = &ogg_set_channels;
	write_buffer = &ogg_write_buffer;
	
	/* << at this point we need to know resolution (snd_fmt),       >>
	 * << channels, and sampling rate of the audio file             >> */
	printf("File: %s, %d Hz, %d bit, %d channels\n", argv[1], sampling_rate, resolution, channels); 

	frame_size = resolution * channels / 8;
	init_alsa(sampling_rate, snd_fmt, channels);
	buf = malloc(BUFSIZE * frame_size);

	set_resolution(resolution);
	set_sampling_rate(sampling_rate);
	set_channels(channels);

	open_stream(argv[1]);
	
	printf("REC!\n");
	while (1) {
		err = snd_pcm_readi(capture_handle, buf, BUFSIZE) ;
		if (err > 0) {
			write_buffer(buf, err * frame_size);
		}
	}
	snd_pcm_close (capture_handle);
	free(buf); buf = NULL;
	return(0);
}

int init_alsa(unsigned int sampling_rate, snd_pcm_format_t snd_fmt, int channels){
	int err;
	snd_pcm_hw_params_t *hw_params;

	if ((err = snd_pcm_open (&capture_handle, device, SND_PCM_STREAM_CAPTURE, 0)) < 0) {
		fprintf (stderr, "cannot open audio device %s (%s)\n", 
				device,
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
		fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_any (capture_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_set_access (capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
		fprintf (stderr, "cannot set access type (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_set_format (capture_handle, hw_params, snd_fmt)) < 0) {
		fprintf (stderr, "cannot set sample format (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_set_rate_near (capture_handle, hw_params, &sampling_rate, 0)) < 0) {
		fprintf (stderr, "cannot set sample rate (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params_set_channels (capture_handle, hw_params, channels)) < 0) {
		fprintf (stderr, "cannot set channel count (%s)\n",
				snd_strerror (err));
		return (1);
	}

	if ((err = snd_pcm_hw_params (capture_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot set parameters (%s)\n",
				snd_strerror (err));
		return (1);
	}

	snd_pcm_hw_params_free (hw_params);

	if ((err = snd_pcm_prepare (capture_handle)) < 0) {
		fprintf (stderr, "cannot prepare audio interface for use (%s)\n",
				snd_strerror (err));
		return (1);
	}

	return 0;
}
